from datetime import datetime
from bs4 import BeautifulSoup
from urllib.parse import urlparse

from .extractor import *
from .generic_extractors import *

import json

import logging

logger = logging.getLogger(__name__)

class GoogleCalendarLinkEventExtractor(Extractor):
    def __init__(self):
        super().__init__()
        self.possible_urls = ["https://calendar.google.com/calendar/", "https://addtocalendar.com/", "https://www.google.com/calendar/event"]


    def guess_image(self, soup, url):
        image = soup.find("meta", property="og:image")

        if image is None:
            for img in soup.select('img'):
                if img.find_parent(name='nav'):
                    continue
                image = img["src"]
                break
        else:
            image = image["content"]

        if image.startswith("/"):
            root_url = "https://" + urlparse(url).netloc + "/"
            image = root_url + image

        return image


    def extract(
        self, content, url, url_human=None, default_values=None, published=False
    ):
        soup = BeautifulSoup(content, "html.parser")

        for ggu in self.possible_urls:

            link_calendar = soup.select('a[href^="' + ggu + '"]')
            if len(link_calendar) != 0:

                gg_cal = GGCalendar(link_calendar[0]["href"])

                if gg_cal.is_valid_event():
                    start_day = gg_cal.start_day
                    start_time = gg_cal.start_time
                    description = gg_cal.description.replace('&nbsp;', '')
                    end_day = gg_cal.end_day
                    end_time = gg_cal.end_time
                    location = gg_cal.location
                    title = gg_cal.title
                    url_human = url

                    self.set_header(url)

                    image = self.guess_image(soup, url)

                    category = None

                    self.add_event(
                        default_values,
                        title=title,
                        category=category,
                        start_day=start_day,
                        location=location,
                        description=description,
                        tags=[],
                        uuids=[url],
                        recurrences=None,
                        url_human=url_human,
                        start_time=start_time,
                        end_day=end_day,
                        end_time=end_time,
                        published=published,
                        image=image,
                    )

                    break


        return self.get_structure()